unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OpenGL, StdCtrls, ExtCtrls;

const
 dt:Single=0.0005;

type
  TForm1 = class(TForm)
    StartBtn: TButton;
    StopBtn: TButton;
    Timer1: TTimer;
    StartBtn1: TButton;
    StopBtn1: TButton;
    procedure StartBtnClick(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure StartBtn1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;
  TLifeArea=array[-11..11,-11..11,-11..11] of Byte;
  TField=array[-101..101,-101..101] of Single;

var
 Form1     : TForm1;
 glDC      : HDC;
 GLContext : HGLRC;
 A,nA,C:TLifeArea;
 F,nF:TField;
 Mode:Byte; {1 - Life game, 2 - Field}

{Discreet Life game procedures}
procedure InitOpenGL;
procedure DoneOpenGL;
procedure DrawPixels;
procedure DrawMargin;
procedure InitLifeArea;
procedure InitLifeMargin;
procedure NextLifeArea;
{Continual Wave model procedures}
procedure DrawField;
procedure InitField;
procedure InitFieldMargin;
procedure NextField;

implementation

procedure Error(Msg:string;Num:integer);
begin
  Msg := Msg+' '+IntToStr(Num);
  MessageBox(0,PChar(Msg),nil,MB_OK);
  PostQuitMessage(0);
end;

procedure InitOpenGL;
var
  pfd: TPixelFormatDescriptor;
  FormatIndex,
  dx,
  dy: integer;
begin
  fillchar(pfd,SizeOf(pfd),0);
  with pfd do
  begin
    nSize      := SizeOf(pfd);
    nVersion   := 1; {The current version of the desccriptor is 1}
    dwFlags    := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL or PFD_DOUBLEBUFFER;
    iPixelType := PFD_TYPE_RGBA;
    cColorBits := 24; {support 24-bit color}
    cDepthBits := 32; {depth of z-axis}
    iLayerType := PFD_MAIN_PLANE;
  end; {with}
  glDC := getDC(Form1.Handle);
  FormatIndex := ChoosePixelFormat(glDC,@pfd);
  if FormatIndex=0 then
   Error('ChoosePixelFormat failed ',GetLastError);
  if not SetPixelFormat(glDC,FormatIndex,@pfd) then
    Error('SetPixelFormat failed ',GetLastError);
  GLContext := wglCreateContext(glDC);
  if GLContext=0 then
    Error('wglCreateContext failed ',GetLastError);
  if not wglMakeCurrent(glDC,GLContext) then
    Error('wglMakeCurrent failed ',GetLastError);
  glMatrixMode(GL_PROJECTION);
  glFrustum(-2,2,-2,2,2,37.0);
  glEnable(GL_DEPTH_TEST);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
  dx:= Form1.Width;
  dy:= Form1.Height;
  glViewPort(0,0,dx,dy);
  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);
  glClearColor(0.0,0.0,0.0,0.0);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
end;

procedure DoneOpenGL;
begin
 wglMakeCurrent(glDC,0);
 wglDeleteContext(GLContext);
end;

procedure DrawPixels;
var
 X,Y,Z:Single;
 i,j,k:Integer;
begin
 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
 glPushMatrix;
 glTranslatef(0.0,0.0,-3.7);
 glRotatef(60,0,1,0);
 glPushMatrix;
 glBegin(GL_POINTS);
 glColor3f(0.999,0.99,0.9);
 Randomize;
 for i:=-10 to 10 do
  for j:=-10 to 10 do
   for k:=-10 to 10 do
    if A[i,j,k]>0 then
    begin
     if C[i,j,k]=26 then
      glColor3f(1,0,0)
     else
      glColor3f(C[i,j,k]/26,0.8,0.7);
     glVertex3f(i/10,j/10,k/10);
    end;
 glEnd;
 glColor3f(0.4,0.8,0.6);
 DrawMargin;
 glPopMatrix;
 glPopMatrix;
 SwapBuffers(glDC);
end;

procedure InitLifeArea;
var
 i,j,k,n:Integer;
begin
 for n:=1 to 1000 do
 begin
  i:=Random(10)-Random(10);
  j:=Random(10)-Random(10);
  k:=Random(10)-Random(10);
  A[i,j,k]:=1;
 end;
end;

procedure InitLifeMargin;
var
 i,j,k:Integer;
begin
 for i:=-11 to 11 do
  for j:=-11 to 11 do
   A[i,j,11]:=0;
 for i:=-11 to 11 do
  for j:=-11 to 11 do
   A[i,j,-11]:=0;
 for j:=-11 to 11 do
  for k:=-11 to 11 do
   A[11,j,k]:=0;
 for j:=-11 to 11 do
  for k:=-11 to 11 do
   A[-11,j,k]:=0;
 for i:=-11 to 11 do
  for k:=-11 to 11 do
   A[i,11,k]:=0;
 for i:=-11 to 11 do
  for k:=-11 to 11 do
   A[i,-11,k]:=0;
end;

procedure NextLifeArea;
var
 i,j,k:Integer;
 S:Byte;
begin
 for i:=-10 to 10 do
  for j:=-10 to 10 do
   for k:=-10 to 10 do
   begin
    S:=A[i-1,j-1,k+1] + A[i-1,j,k+1] + A[i-1,j+1,k+1] +
       A[i,j-1,k+1]   + A[i,j,k+1]   + A[i,j+1,k+1]   +
       A[i+1,j-1,k+1] + A[i+1,j,k+1] + A[i+1,j+1,k+1];
    S:=S +
       A[i-1,j-1,k]   + A[i-1,j,k]   + A[i-1,j+1,k]   +
       A[i,j-1,k]     + A[i,j+1,k]   +
       A[i+1,j-1,k]   + A[i+1,j,k]   + A[i+1,j+1,k];
    S:=S +
       A[i-1,j-1,k-1] + A[i-1,j,k-1] + A[i-1,j+1,k-1] +
       A[i,j-1,k-1]   + A[i,j,k-1]   + A[i,j+1,k]     +
       A[i+1,j-1,k-1] + A[i+1,j,k-1] + A[i+1,j+1,k-1];
    if ((A[i,j,k]=1) and (S in [7..13])) then
    begin
     nA[i,j,k]:=1;
     C[i,j,k]:=13
    end
    else
    begin
     nA[i,j,k]:=0;
     C[i,j,k]:=0;
    end;
    if (S in [9..11]) then
    begin
     nA[i,j,k]:=1;
     C[i,j,k]:=26
    end;
   end;
 for i:=-10 to 10 do
  for j:=-10 to 10 do
   for k:=-10 to 10 do
   begin
    A[i,j,k]:=nA[i,j,k];
   end;
 InitLifeMargin;
end;

procedure DrawMargin;
begin
 {glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
 glPushMatrix;
 glTranslatef(0.0,0.0,-3.7);
 glRotatef(60,0,1,0);
 glPushMatrix;
 glColor3f(0.999,0.99,0.9);}
 glBegin(GL_LINE_LOOP);
  glVertex3f(1,1,1);
  glVertex3f(-1,1,1);
  glVertex3f(-1,1,-1);
  glVertex3f(1,1,-1);
 glEnd;
 glBegin(GL_LINE_LOOP);
  glVertex3f(1,-1,1);
  glVertex3f(-1,-1,1);
  glVertex3f(-1,-1,-1);
  glVertex3f(1,-1,-1);
 glEnd;
 glBegin(GL_LINES);
  glVertex3f(1,-1,1);
  glVertex3f(1,1,1);
  glVertex3f(-1,-1,1);
  glVertex3f(-1,1,1);
  glVertex3f(-1,-1,-1);
  glVertex3f(-1,1,-1);
  glVertex3f(1,-1,-1);
  glVertex3f(1,1,-1);
 glEnd;
 {glPopMatrix;
 glPopMatrix;
 SwapBuffers(glDC);}
end;

procedure InitField;
var
 i,j:Integer;
begin
 Randomize;
 for i:=-100 to 100 do
  for j:=-100 to 100 do
   F[i,j]:=Exp(-0.01*(i*i + j*j));
 for i:=-101 to 101 do
 begin
  F[i,-101]:=0;
  F[i,101]:=0
 end;
 for j:=-101 to 101 do
 begin
  F[-101,j]:=0;
  F[101,j]:=0
 end;
end;

procedure InitFieldMargin;
var
 i,j:Integer;
begin
 for i:=-101 to 101 do
 begin
  F[i,-101]:=0;
  F[i,101]:=0
 end;
 for j:=-101 to 101 do
 begin
  F[-101,j]:=0;
  F[101,j]:=0
 end;
end;

procedure NextField;
var
 i,j:Integer;
 S,pF:Single;
begin
 for i:=-100 to 100 do
  for j:=-100 to 100 do
  begin
   S:=F[i-1,j-1] + F[i-1,j] + F[i-1,j+1] +
      F[i,j-1]   {+ F[i,j]} + F[i,j+1]   +
      F[i+1,j-1] + F[i+1,j] + F[i+1,j+1];
   pF:=S*S*(16-S*S);
   nF[i,j]:=F[i,j] + pF*dt;
  end;
 for i:=-100 to 100 do
  for j:=-100 to 100 do
   F[i,j]:=nF[i,j];
 InitFieldMargin;
end;

procedure DrawField;
var
 i,j:Integer;
 C:Single;
begin
 glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
 glPushMatrix;
 glTranslatef(0.0,0.0,-4.1);
 //glRotatef(60,0,1,0);
 glPushMatrix;
 Randomize;
 glBegin(GL_POINTS);
 for i:=-100 to 100 do
  for j:=-100 to 100 do
  begin
   C:=Abs(F[i,j]);
   if C<1 then
    glColor3f(0,0,C);
   if (C>1) and (C<10) then
    glColor3f(0,C,0);
   if (C>10) and (C<100) then
    glColor3f(C,0,0);
   glVertex2f(i/100,j/100);
  end;
 glEnd;
 glPopMatrix;
 glPopMatrix;
 SwapBuffers(glDC);
end;

{$R *.dfm}

procedure TForm1.StartBtnClick(Sender: TObject);
begin
 InitLifeArea;
 InitOpenGL;
 DrawPixels;
 Mode:=1;
 Timer1.Enabled:=True;
end;

procedure TForm1.StopBtnClick(Sender: TObject);
begin
 DoneOpenGL;
 Exit;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
 if Mode=1 then
 begin
  NextLifeArea;
  DrawPixels;
 end
 else
 begin
  NextField;
  DrawField;
 end;
end;

procedure TForm1.StartBtn1Click(Sender: TObject);
begin
 InitField;
 InitOpenGL;
 DrawField;
 Mode:=2;
 Timer1.Enabled:=True;
end;

end.
